% W = connectivityadjacency(W1,W2,X1,X2,corr12,corr21,downsample)
%    Build connectivity adjacency, contains adjacency from mesh 1 and 2,
%    plus the adjacency of connectivities between mesh 1 and 2
%      W1         - adjacency matrix of mesh 1 (n1 x n1)
%      W2         - adjacency matrix of mesh 2 (n2 x n2)
%      X1         - vertex position of mesh 1
%      X2         - vertex position of mesh 2
%      corr12     - correspondence from mesh 1 onto mesh 2
%      corr21     - correspondence from mesh 2 onto mesh 1
%      downsample - downsample connectivities (default: 5)
%
function W = connectivityadjacency(W1,W2,X1,X2,corr12,corr21,downsample)

    if nargin<7;  downsample = 5;  end;
    
    if size(corr12,1)==1;  corr12 = corr12';  end;
    if size(corr21,1)==1;  corr21 = corr21';  end;

    n1  = size(X1,1);
    n2  = size(X2,1);
    eps = 1e-6; % small number

    % Indices and weights of W1,W2
    [i1,j1,w1] = find(W1);
    [i2,j2,w2] = find(W2); i2=i2+n1; j2=j2+n1;

    if ~isempty(corr21)
        % Indices and weights of connectivities
        links12 = (sqrt(sum((X1-X2(corr12,:)).^2,2)) + eps).^-1;
        links21 = (sqrt(sum((X2-X1(corr21,:)).^2,2)) + eps).^-1;

        icorr = [(1:downsample:n1)';           corr12(1:downsample:end)+n1;  (1:downsample:n2)'+n1;      corr21(1:downsample:end); ]; % remains symmetric
        jcorr = [corr12(1:downsample:end)+n1;  (1:downsample:n1)';           corr21(1:downsample:end);   (1:downsample:n2)'+n1;    ];
        wcorr = [links12(1:downsample:end);    links12(1:downsample:end);    links21(1:downsample:end);  links21(1:downsample:end);];
    else
        % Indices and weights of connectivities
        links12 = (sqrt(sum((X1-X2(corr12,:)).^2,2)) + eps).^-1;

        icorr = [(1:downsample:n1)';           corr12(1:downsample:end)+n1;]; % remains symmetric
        jcorr = [corr12(1:downsample:end)+n1;  (1:downsample:n1)';         ];
        wcorr = [links12(1:downsample:end);    links12(1:downsample:end);  ];
    end

    % Adjacency matrix of correspondence graph
    W = sparse([i1; i2; icorr]',[j1; j2; jcorr]',[w1; w2; wcorr]',n1+n2,n1+n2); % Bidirectional links

end
