% displaycorr(X1,F1,X2,F2,corr12,corr21,E1,E2)
%    Display correspondences
%
function displaycorr(X1,F1,X2,F2,corr12,corr21,E1,E2)

    do_interactive = 1;
    
    % mouseclick callback
    if do_interactive
        datacursormode off;
        data.mousehandle = datacursormode(gcf);
        set(data.mousehandle,'UpdateFcn',@highlight,'SnapToDataVertex','on','DisplayStyle','window'); % callback function
        datacursormode on;
    end

    data.marker = []; % initially empty handles to plot3 markers
    data.X1     = X1;
    data.F1     = F1;
    data.X2     = X2;
    data.F2     = F2;
    data.corr12 = corr12;
    data.corr21 = corr21;
    
    n1 = size(X1,1);
    n2 = size(X2,1);

    clf; set(gcf,'color','w');
    %colormap jet;
    lim = [X1;X2]; lim = [min(lim(:,1)) max(lim(:,1)) min(lim(:,2)) max(lim(:,2)) min(lim(:,3)) max(lim(:,3))];

    if nargin>7
        idx = [1 ceil(size(E1,2)/2) size(E1,2)];
        sE2 = bsxfun(@rdivide,bsxfun(@minus,E2,min(E2)),range(E2)); % for colormap between 0..1
        brightness = .1;
        colormap(sE2(:,idx)*(1-brightness)+brightness);
        C2 = ((1:n2)-1)/(n2-1);
        C1 = (corr12-1)/(n2-1);
    else
        C2  = X2(:,3);
        C1  = X2(corr12,3);
        colormap jet;
    end
    
    % Deformatin of mesh 1
    data.hsubplot(1) = subplot(1,2,1);
    data.hmesh(1) = trisurf(F1,X1(:,1),X1(:,2),X1(:,3),C1); shading interp; caxis([0 1]); daspect([1 1 1]); view(global_viewaxis); material dull; axis(lim); camlight;
    text(mean(X1(:,1)),mean(X1(:,2)),min(X1(:,3)),'Mesh 1');

    % Deformatin of mesh 2
    data.hsubplot(2) = subplot(1,2,2);
    data.hmesh(2) = trisurf(F2,X2(:,1),X2(:,2),X2(:,3),C2); shading interp; caxis([0 1]); daspect([1 1 1]); view(global_viewaxis); material dull; axis(lim); camlight;
    text(mean(X2(:,1)),mean(X2(:,2)),min(X2(:,3)),'Mesh 2');
    

    data.linkprop = linkprop(data.hsubplot,{'cameraposition','cameraupvector','cameratarget','cameraviewangle'});
    set(gcf, 'UserData', data);

    % Illuminate both sides
    subplot(1,2,1); view(global_viewaxis-[180,0]); camlight;  subplot(1,2,2); camlight;  view(global_viewaxis);
    
    % Callback function
    function txt = highlight(obj,event)
        if strcmp(class(event),'graphics.datatipevent')
            data        = get(gcf,'UserData'); % get previous marker
            idx         = get(event,'DataIndex'); % get vertex index of cursor
            cursorinfo  = getCursorInfo(data.mousehandle); % handle
            clickedmesh = find(cursorinfo.Target == data.hmesh);
            if ~isempty(data.marker); delete(data.marker(1)); delete(data.marker(2)); end; % delete previous marker

            switch clickedmesh
                case 1
                    x1 = data.X1(idx,:);
                    x2 = data.X2(corr12(idx),:);
                case 2
                    x1 = data.X1(corr21(idx),:);
                    x2 = data.X2(idx,:);
                otherwise txt = ''; return;
            end

            % Draw markers on mesh
            data.hsubplot(1) = subplot(1,2,1);
            hold on; data.marker(1) = plot3(x1(1),x1(2),x1(3),'k+','MarkerSize',50,'LineWidth',3); hold off;

            data.hsubplot(2) = subplot(1,2,2);
            hold on; data.marker(2) = plot3(x2(1),x2(2),x2(3),'k+','MarkerSize',50,'LineWidth',3); hold off;

            set(gcf,'UserData',data); % remember our markers

            % Display distance between correspondence point
            d = sqrt(sum((x1-x2).^2,2));
            txt = ['dist: ' num2str(clickedmesh)];
        else
            txt = '';
        end
    end
    
    

end
