% displaycorrespondences(X1,F1,X2,F2,corr12,corr21)
%    Display correspondences
%
function displaycorrespondences(X1,F1,X2,F2,corr12,corr21,E1,E2)

    n1 = size(X1,1);
    n2 = size(X2,1);
    viewaxis = [90,0];
    
    clf;
    colormap jet;
    
    if nargin>7
        idx = [1 ceil(size(E1,2)/2) size(E1,2)];
        sE2 = bsxfun(@rdivide,bsxfun(@minus,E2,min(E2)),range(E2)); % for colormap between 0..1
        brightness = .1;
        colormap(sE2(:,idx)*(1-brightness)+brightness);
        C2 = ((1:n2)-1)/(n2-1);
        C1 = (corr12-1)/(n2-1);
    else
        C2  = X2(:,3);
        C1  = X2(corr12,3);
    end
    
    lim = [X1;X2]; lim = [min(lim(:,1)) max(lim(:,1)) min(lim(:,2)) max(lim(:,2)) min(lim(:,3)) max(lim(:,3))];
    
    % Mesh 2
    h(1) = subplot(2,2,1);
    trisurf(F2,X2(:,1),     X2(:,2),     X2(:,3),     C2); shading interp; daspect([1 1 1]); caxis([0 1]); view(viewaxis); material dull; camlight; axis off; axis(lim);
    text(mean(X2(:,1)),mean(X2(:,2)),min(X2(:,3)),'Mesh 2');
    
    % Mesh 1 deformed onto mesh 2
    h(2) = subplot(2,2,2);
    trisurf(F1,X2(corr12,1),X2(corr12,2),X2(corr12,3),C1); shading interp; daspect([1 1 1]); caxis([0 1]); view(viewaxis); material dull; camlight; axis off; axis(lim);
    text(mean(X2(corr12,1)),mean(X2(corr12,2)),min(X2(corr12,3)),'Deformed Mesh 1');
    
    obj = linkprop(h,{'cameraposition','cameraupvector','cameratarget','cameraviewangle'});
    set(gcf,'UserData',obj);

    % Mesh 1 with correspondence field
    subplot(2,1,2);
    dX1 = X2(corr12,:) - X1;
    trisurf(F1,X1(:,1),X1(:,2),X1(:,3),C1); shading interp; daspect([1 1 1]); caxis([0 1]); view(viewaxis); material dull; camlight; axis off; axis(lim);
    text(mean(X1(:,1)),mean(X1(:,2)),min(X1(:,3)),'Mesh 1 with displacements');
    s = 50;
    hold on; quiver3(X1(1:s:end,1),X1(1:s:end,2),X1(1:s:end,3), dX1(1:s:end,1),dX1(1:s:end,2),dX1(1:s:end,3), 0); hold off;
    

end