% displayembeddings(E,F,dimidx,downsample)
%    Display spectral embedding as point cloud or as triangulated mesh
%    E          - spectral embedding (n x k)
%    F          - optional triangulation to display as mesh (default: [])
%    dimidx     - dimensins to use in 2D or 3D (default: [2 3 4])
%    downsample - downsample nb of points to show if using point cloud
%
function displayembeddings(E,F,dimidx,downsample,linespec)

    if nargin<2;  F = [];            end;
    if nargin<3;  dimidx = [2 3 4];  end;
    if nargin<4;  downsample = 10;   end;
    if nargin<5;  linespec = 'b.';   end;
    
    ndims = numel(dimidx);

    if ~isempty(F) && ndims == 3
        
        C = E(:,dimidx(3));
        trisurf(F,E(:,dimidx(1)),E(:,dimidx(2)),E(:,dimidx(3)),C); daspect([1 1 1]); shading interp; material dull;
        xlabel(['e' num2str(dimidx(1))]);
        ylabel(['e' num2str(dimidx(2))]);
        zlabel(['e' num2str(dimidx(3))]);
        
    else
        
        s = downsample;
        switch ndims
            case 2
                plot( E(1:s:end,dimidx(1)),E(1:s:end,dimidx(2)),linespec);
                xlabel(['e' num2str(dimidx(1))]);
                ylabel(['e' num2str(dimidx(2))]);
            case 3
                plot3(E(1:s:end,dimidx(1)),E(1:s:end,dimidx(2)),E(1:s:end,dimidx(3)),linespec);
                xlabel(['e' num2str(dimidx(1))]);
                ylabel(['e' num2str(dimidx(2))]);
                zlabel(['e' num2str(dimidx(3))]);
        end
        
    end
    

end
