% displayspectrums(X1,F1,X2,F2,E1,E2)
%   Display spectral embeddings
%
function displayspectrums(X1,F1,X2,F2,E1,E2,show_isolines,show_eigenvectors,show_embeddings,downsample,showtitles)

    if nargin<7;  show_isolines     = 0;    end;
    if nargin<8;  show_eigenvectors = 1;    end;
    if nargin<9;  show_embeddings   = 1;    end;
    if nargin<10; downsample        = 200;  end;
    if nargin<10; showtitles        = 1;  end;

    nrows = 0;
    if show_eigenvectors; nrows = nrows+2;  end;
    if show_embeddings;   nrows = nrows+1;  end;

    k        = size(E1,2);
    idx      = 1:k; % dimensions to display
    ndims    = numel(idx);
    
    lim = [X1;X2]; lim = [min(lim(:,1)) max(lim(:,1)) min(lim(:,2)) max(lim(:,2)) min(lim(:,3)) max(lim(:,3))];
    
    clf;
    colormap jet;
    
    if show_eigenvectors
        colormap jet;
        for i=1:ndims
            crange = [min([E1(:,idx(i)); E2(:,idx(i))]) max([E1(:,idx(i)); E2(:,idx(i))])];
            isovalues = linspace(min([E1(:,idx(i)); E2(:,idx(i))]),max([E1(:,idx(i)); E2(:,idx(i))]), 10);
            
            h(i) = subplot(nrows,ndims,i);
            if ~show_isolines; trisurf(F1,X1(:,1),X1(:,2),X1(:,3),E1(:,idx(i)));
            else               isolines(X1,F1,E1(:,idx(i)),isovalues);           end;
            daspect([1 1 1]); caxis(crange); shading interp; material dull; axis(lim); view(global_viewaxis); zoom(1.35); axis off; camlight;
            if showtitles;  text(mean(X1(:,1)),mean(X1(:,2)),min(X1(:,3)),['e' num2str(i)]);  end;
            view(global_viewaxis+[180 0]); camlight; view(global_viewaxis);

            h(i+ndims) = subplot(nrows,ndims,i+ndims);
            if ~show_isolines; trisurf(F2,X2(:,1),X2(:,2),X2(:,3),E2(:,idx(i)));
            else               isolines(X2,F2,E2(:,idx(i)),isovalues);           end;
            daspect([1 1 1]); caxis(crange); shading interp; material dull; axis(lim); view(global_viewaxis); zoom(1.35); axis off; camlight;
            if showtitles;  text(mean(X2(:,1)),mean(X2(:,2)),min(X2(:,3)),['e' num2str(i)]);  end;
            view(global_viewaxis+[180 0]); camlight; view(global_viewaxis);
        end
        obj = linkprop(h,{'cameraposition','cameraupvector','cameratarget','cameraviewangle'});
        set(gcf,'UserData',obj);
    end
    
    if show_embeddings
        if show_eigenvectors; pos = 3;  else pos = 1;  end;
        subplot(nrows,1,pos);
                  displayembeddings(E1,[],[1 2 3],downsample,'b.'); daspect([1 1 1]);
        hold on;  displayembeddings(E2,[],[1 2 3],downsample,'r.'); daspect([1 1 1]);  hold off;
        view(2);
    end

end
