% Fp = interpolate(Xp,
%   Interpolate feature value from neighbors
%     Xp   - where to interpolate feature value
%     corr - closest corresponding sampled point
%     X    - where features are known
%     A    - weighted adjacency matrix between sampled points (inverse distance)
%     F    - feature values at sampled points
%
function Fp = interpolate(Xp,corr,X,W,F)

    % Gaussian Interpolation
    %   wi = exp(-(p - xi)^2 / 2*sigma^2)
    %   fp = sum( wi * fi ) / sum(wi)

    n     = size(W,1);  % number of sample points
    np    = size(Xp,1); % number of interpolated points
    ndims = size(X,2);  % sampled coordinate dimensions
    eps   = 1e-6;
    
    % find average edge length
    %[i,j,w] = find(W);
    %sigma = mean(w.^-1-eps);
    %sigma = max([mean(w.^-1-eps)  max(w.^-1-eps)/3]);
    %sigma = 0.015895; % max of all possibles sigma in each pairwise registration
    %disp(['sigma: ' num2str(sigma)]);
    
    A = 1*(W>0); % binary adjacency matrix
    A = speye(n) + A; % add itself on the diagonal
    
    % Use corresponding points
    A = A(corr,:);
    [i,j] = find(A);
    
    % Compute norm of dist(pi,xj)
    r = sqrt(sum((Xp(i,:) - X(j,:)).^2,2)); % R_ij = dist(pi,xj)

    % find maximal distance between p and x
    sigma = max(r);

    % Gaussian kernel
    r = exp(-r.^2 / (2*sigma^2));           % R_ij = exp(-dist^2 / 2*sigma^2)
    R = sparse(i,j,r,np,n);
    
    % Normalized Gaussian weights
    %D     = spdiags(sum(R,2),0,speye(np));     % row sum of all weights
    d     = sum(R,2);  dinv  = d.^-1;  dinv(d<eps) = 0;
    Dinv  = spdiags(dinv,0,speye(np)); % row sum of all weights (inverse)
    W = Dinv * R;                              % normalized weights (horz. multiplication)

    % Interpolate
    Fp = W * F;

end
