% Show differences in deforming mesh using nearest points and interpolated
% points
%    M1     - coordinates where to interpolate points
%    corr12 - correspondences from mesh 1 onto mesh 2
%    M2     - known values on mesh 2
%    W2     - connectivity of mesh 2
%    X2     - coordinates at known value
%    X1p    - interpolated coordinates values from mesh 2
%
function X1p = showinterpolation(X1,F1,X2,F2,M1,M2,corr12,corr21,W1,W2)

    clf;
    colormap jet;
    lim = [X1;X2]; lim = [min(lim(:,1)) max(lim(:,1)) min(lim(:,2)) max(lim(:,2)) min(lim(:,3)) max(lim(:,3))];
    lim = [X1;X2]; lim = [min(lim(:,1)) max(lim(:,1)) min(lim(:,2)) min(lim(:,2))+.2*range(lim(:,2)) min(lim(:,3)) min(lim(:,3))+.2*range(lim(:,3))];
    
    % Original second mesh
    trisurf(F2,X2(:,1),X2(:,2),X2(:,3),M2(:,1)); shading interp; daspect([1 1 1]); view(-90,0); material dull; axis(lim); camlight;
    
    % Closest point coordinate
    X1n = X2(corr12,:);
    %trisurf(F1,X1n(:,1),X1n(:,2),X1n(:,3),-M1(:,1)); shading interp; daspect([1 1 1]); view(-90,0); material dull; axis(lim); camlight;
    
    % Interpolated coordinates
    X1p = interpolate(M1,corr12,M2,W2,X2);
    hold on; trisurf(F1,X1p(:,1),X1p(:,2),X1p(:,3),-M1(:,1)); shading interp; daspect([1 1 1]); view(-90,0); material dull; axis(lim); hold off;
    
    alpha(.2);
    
    % Show differences between nearest point and interpolated position
    idx = 1:50:size(M1,1);
    hold on; plot3(X1n(idx,1),X1n(idx,2),X1n(idx,3), 'bx'); hold off;
    hold on; plot3(X1p(idx,1),X1p(idx,2),X1p(idx,3), 'bo'); hold off;
    hold on; quiver3(X1n(idx,1),X1n(idx,2),X1n(idx,3),  X1p(idx,1)-X1n(idx,1),X1p(idx,2)-X1n(idx,2),X1p(idx,3)-X1n(idx,3), 0); hold off;

end
