% E2 = signflip(E1,E2,X1,X2)
%   Flip sign of eigenvectors E2 so they match E1
%   E1 - shape eigenvectors of mesh 1 (n x k)
%   E2 - shape eigenvectors of mesh 2 (n x k)
%   X1 - vertices of mesh 1 (n x ndims)
%   X2 - vertices of mesh 2 (n x ndims)
%
%   Pair closest points in space between X1,X2 - check the sign coherence
%   between E1,E2 at these pairs of closest points
%
function E2 = signflip(E1,E2,X1,X2)

    % Subsample X1,X2
    downsample = max(ceil(min(size(X1,1),size(X2,1)) / 100), 10);
    sX1 = X1(1:downsample:end,:);
    sX2 = X2(1:downsample:end,:);
    sE1 = E1(1:downsample:end,:);
    sE2 = E2(1:downsample:end,:);
    
    % Find closest points in space
    tree = kdtree(sX2);
    corr = kdtree_closestpoint(tree, sX1); % all corresponding point from sX1 onto sX2
    
    % Check sign
    dotp  = diag( sign(sE2(corr,:))' * sign(sE1) ); % dot product between corresponding E2 and E1
    signf = 1 - 2*(dotp<0);  % column vector telling if there is a signflip (-1) or not (+1)
    
    % Flip sign
    E2 = bsxfun(@times,E2,signf');  % Align E2 in the same direction as E1

end
