% W = weightedadjacency(X,F)
%   Compute weighted adjacency matrix
%   X - vertex position (n x ndim)
%   F - triangulation (n x 3, vertex indices for each triangle)
%   W - weighted adjacency matrix (n x n)
%       if nodes i and j are connected, W(i,j) gives their affinity
%       affinity is the inverse distance W(i,j) = dist(X(i,:) - X(j,:)))^-1
%
function W = weightedadjacencynormal(X,F)

    N = normal(X,F);

    % compute weights for all links (euclidean distance)
    wdist   = [sum((X(F(:,1),:)-X(F(:,2),:)).^2,2); ...
               sum((X(F(:,1),:)-X(F(:,3),:)).^2,2); ...
               sum((X(F(:,2),:)-X(F(:,1),:)).^2,2); ...
               sum((X(F(:,2),:)-X(F(:,3),:)).^2,2); ...
               sum((X(F(:,3),:)-X(F(:,1),:)).^2,2); ...
               sum((X(F(:,3),:)-X(F(:,2),:)).^2,2)].^0.5;
           
    % compute weights for all links (euclidean distance)
    wnormal = [sum((N(F(:,1),:)-N(F(:,2),:)).^2,2); ...
               sum((N(F(:,1),:)-N(F(:,3),:)).^2,2); ...
               sum((N(F(:,2),:)-N(F(:,1),:)).^2,2); ...
               sum((N(F(:,2),:)-N(F(:,3),:)).^2,2); ...
               sum((N(F(:,3),:)-N(F(:,1),:)).^2,2); ...
               sum((N(F(:,3),:)-N(F(:,2),:)).^2,2)].^0.5;
           
    wdist   = wdist   / mean(wdist);
    wnormal = wnormal / mean(wnormal);
           
    eps     = 1e-6; % penalize small distances (avoid division by zero)
    %weights = (weights + eps).^-1;
    %weights = (wdist.*wnormal + eps).^-1;
    weights = (wdist + wnormal + eps).^-1;
    %sigma = ((max(weights)))/3;
    %weights = exp(-weights.^2 / (2*sigma^2));
    
    % remove duplicated edges
    rows = [F(:,1); F(:,1); F(:,2); F(:,2); F(:,3); F(:,3)];
    cols = [F(:,2); F(:,3); F(:,1); F(:,3); F(:,1); F(:,2)];
    [rc,idx] = unique([rows,cols], 'rows','first');
    weights = weights(idx);

    % fill adjacency matrix
    W = sparse(rc(:,1), rc(:,2), weights);
    
end

